<?php

declare(strict_types=1);

/*
 * Copyright (c) 2017-2023 François Kooman <fkooman@tuxed.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

namespace fkooman\OAuth\Server;

use DateTimeImmutable;

class Authorization
{
    private string $authKey;
    private string $userId;
    private string $clientId;
    private Scope $scope;
    private DateTimeImmutable $authorizedAt;
    private DateTimeImmutable $expiresAt;
    private DateTimeImmutable $lastUsed;

    public function __construct(string $authKey, string $userId, string $clientId, Scope $scope, DateTimeImmutable $authorizedAt, DateTimeImmutable $expiresAt, DateTimeImmutable $lastUsed)
    {
        $this->authKey = $authKey;
        $this->userId = $userId;
        $this->clientId = $clientId;
        $this->scope = $scope;
        $this->authorizedAt = $authorizedAt;
        $this->expiresAt = $expiresAt;
        $this->lastUsed = $lastUsed;
    }

    public function authKey(): string
    {
        return $this->authKey;
    }

    public function userId(): string
    {
        return $this->userId;
    }

    public function clientId(): string
    {
        return $this->clientId;
    }

    public function scope(): Scope
    {
        return $this->scope;
    }

    public function authorizedAt(): DateTimeImmutable
    {
        return $this->authorizedAt;
    }

    public function expiresAt(): DateTimeImmutable
    {
        return $this->expiresAt;
    }

    public function lastUsed(): DateTimeImmutable
    {
        return $this->lastUsed;
    }
}
